(* :Title: Guides.m *)

(* :Author:
	Andrew Hunt, andy@wolfram.com
	Greg Martel, gregm@wolfram.com
*)

(* :Mathematica Version: 6.0 *)

(* :Package Version: $Revision: 1.37.2.2 $ $Date: 2016/10/28 15:34:36 $ *)
                  
(* :Copyright: (c) 2005, Wolfram Research, Inc. All rights reserved. *)

(* :Summary: XMLTransform for Documentation Guide pages *)

(* :ToDo: 

*)


XMLTransform[{


(***** Match Notebook expression *****)


(***** Cell Definitions *****)
{Cell, "AnchorBar" | "History"} :> Unevaluated[Sequence[]],

{Cell, "RootSearch" | "RootTitle" | "RootFooterSep"} :> Unevaluated[Sequence[]],

{Cell, "RootFooterRedLink"} :> 
XMLElement["span",{},{Recurse[]}],

{Cell, "RootHeader"} :> Unevaluated[Sequence[]],

{Cell, "RootFooter"} :> Module[{allLinks,shadowbar,language},
	language=GetParameter["language"];
	allLinks=Cases[SelectSelf[], Cell[_, "RootFooterRedLink", ___], Infinity];
	
	XMLElement["div", {"class" -> "rootfooter"}, {XMLElement[
   "ul", {}, {XMLElement[
     "li", {}, {XMLElement[
       "a", {"href" -> 
         "/language/guide/AlphabeticalListing.html"}, {Transmogrify[allLinks[[1]]]}]}], 
    XMLElement[
     "li", {}, {XMLElement[
       "a", {"href" -> 
         "http://reference.wolfram.com/legacy/"}, {Transmogrify@Localized["Legacy Documentation", language]}]}]}], 
    XMLElement[
   "ul", {}, {XMLElement[
     "li", {}, {XMLElement[
       "a", {"href" -> 
         "/language/guide/StandardExtraPackages.html"}, {Transmogrify[allLinks[[2]]]}]}],
    XMLElement[
     "li", {}, {XMLElement[
       "a", {"href" -> 
         "http://demonstrations.wolfram.com/"}, {Transmogrify[allLinks[[5]]]}]}]}], 
    XMLElement[
   "ul", {}, {XMLElement[
     "li", {}, {XMLElement[
       "a", {"href" -> 
         "http://www.wolfram.com/products/"}, {Transmogrify[allLinks[[3]]]}]}], 
    XMLElement[
     "li", {}, {XMLElement[
       "a", {"href" -> 
         "http://www.wolfram.com/language/"}, {Transmogrify@Localized["More About Mathematica", language]}]}]}]}]

],

{Cell, "RootLinkText"}:>  Recurse[],

{Cell, "Picture"} :> 
transmogrifyImage[ToFileName[{"Files", GetFileNameBase[]},
    IncrementCounter["Image"] <> ".gif"],
    Inline -> True, CropImage -> False
], 


{Cell,  "2ColumnTable" | "3ColumnTable" | "2ColumnTableMod" | "3ColumnTableMod" | "3ColumnIllustratedTableMod"} :>
  DIV[ {"class" -> "FunctionTable", "name" -> (GetStyle[]/.None->"FIXME")}, 
  {{Recurse[] }}
], 


{Cell, "GuideTOCTitle"} :> 
H1[{},  {Recurse[]}],  


{Cell, "GuideTOCLink"} :> XMLElement["h2", {}, {Recurse[]}],


{Cell, "GuideTOCAbstract"} :>
P[{"class" -> "intro"}, {Recurse[]}],

{Cell, "RootItemSectionTitle"} :> 
Module[{},
    XMLElement["div", {"class"->"podheader icon"<>IncrementCounter["rootitemcounter"], "id"->"icon"<>GetCounter["rootitemcounter"]}, {XMLElement["span", {"class"->"title"}, {Recurse[]}]}]
  ],

{CellGroupData, "RootItemSectionTitle"} :> 
Module[{tilecolor},
	tilecolor = Switch[IncrementCounter["RootItemSectionTitle"], "1"|"2"|"3"|"4", "redbg", "5"|"6"|"7"|"8"|"9", "orangebg", "10"|"11"|"12"|"13"|"14"|"15"|"16", "bluebg", "17"|"18", "greenbg", "19"|"20"|"21", "tealbg", ___, "redbg"];
    XMLElement["div", {"class"->"thumb "<>tilecolor}, {Transmogrify[First[SelectChildren[]]], XMLElement["div", {"class"->"links-list"}, {Transmogrify[Rest[SelectChildren[]]]}]}]
  ],

{CellGroupData, "RootItem"} :> 
Module[{},
    XMLElement["ul", {}, {Recurse[]}]
  ],
 
{Cell, "RootItem"} :> 
Module[{},
    XMLElement["li", {}, {Recurse[]}]
  ],

{Cell, "RootItemDelimiter"} :>
Module[{},
    XMLElement["div", {"class"->"divider"}, {XML`RawXML[""]}]
  ],

{CellGroupData, "RootSection"} :> 
Module[{id, inc1, inc2, iconName, caption, rootnumber},
  	inc1 = IncrementCounter["RootIcon"];
  	inc2 = IncrementCounter["RootSection"];
    iconName = "/mathematicaImages/RootIcon"<>inc1<>".gif";
    id = "scenarios" <> inc2;
    caption = GetParameter["rootCaptions"][[ToExpression[inc2]]];
    rootnumber = {"1"->"one","2"->"two",
    "3"->"three","4"->"four","5"->"five",
    "6"->"six","7"->"seven","8"->"eight"};

    XMLElement["div", {"class"->"accordion " <>(inc2 /. rootnumber)}, {Recurse[]}]
  ],
  
{Cell, "RootSection"} :>
  Module[{id, inc1, inc2, iconName, caption, rootnumber},
  	inc1 = GetCounter["RootIcon"];
  	inc2 = GetCounter["RootSection"];
    iconName = "/mathematicaImages/RootIcon"<>inc1<>".gif";
    id = "scenarios" <> inc2;
    caption = GetParameter["rootCaptions"][[ToExpression[inc2]]];
    rootnumber = {"1"->"one","2"->"two",
    "3"->"three","4"->"four","5"->"five",
    "6"->"six","7"->"seven","8"->"eight"};

    XMLElement["h2", {"class"->"accordion-header " <>(inc2 /. rootnumber)}, {Transmogrify[Cases[SelectSelf[], Cell[a___,"RootSectionTitle", b___], Infinity][[1]]]}]
  ],

{Cell, "RootGrid2Column"} :>
  Module[{row},
    row = SelectNearestDescendants[GridBox][[1, 1]];
    DIV[{"class" -> "RootGrid2Column"}, {
      XMLElement["table", {}, {XMLElement["tr", {}, {
        XMLElement["td", {}, {Transmogrify@First[row]}],
        XMLElement["td", {}, {
          XML`RawXML["<!--#include virtual=\"/searchbar.html\"-->"],
          Transmogrify@Last[row]
        }]
      }] }]
    }]
  ],

{Cell, "RootGrid3Column"} :>
  Module[{row},
    row = SelectNearestDescendants[GridBox][[1, 1, 1]];
    DIV[{"class" -> "RootGrid3Column",  "id"->If[language==="Japanese","ja","en"] }, {
      XMLElement["table", {}, {XMLElement["tr", {}, {
        XMLElement["td", {}, {
          XML`RawXML["<!--#include virtual=\"/searchbar.html\"-->"],
          Transmogrify[row[[1]]]}],
        XMLElement["td", {}, {Transmogrify[row[[2]]]}],
        XMLElement["td", {}, {Transmogrify@Last[row]}]
      }] }]
    }]
  ],

{Cell, "RootGrid"} :>
Block[{flag = GetParameter["flag"], cellflag, cont},
	cellflag = GetStyle[];
	(*cont= SelectChildren[][[1]];*)
  If[flag =!= "RootGuidePage",
    Recurse[],
    DIV[{"class"->"GuideFunctionsSectionBar"}, {
     XMLElement["table", {"class"->"GuideFunctionsSectionTable"}, {
		XMLElement["tr", {}, {XMLElement["td", {}, {Recurse[]}]}]
		}] 
	 }]
   ] ], 


(* {Cell, "RootHeading"}:> DIV[{"class"->"GuideFunctionsSection"}, {
	Span[{"class"->"GuideFunctionsSection"}, {Recurse[]}]
}], *)

{Cell, "RootHeading"}:> XMLElement["div", {"class"->"GuideFunctionsSection"}, {
	Span[{"class"->"GuideFunctionsSection"}, {Recurse[]}]
}],

{BoxData}:> Recurse[],

{Cell, "GuideTitle"} :> 
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    H1[{}, {Recurse[] } ],
    DIV[{}, {}]
  ]
],

{Cell, "GuideTitleGrid"} :>  Block[{objgrid = True}, 
  DIV[{"class"->"GuideTitleGrid"}, {Recurse[]}] ],

{Cell, "IntroLinkCell"} :> Sequence[],
(*
XMLObject["Comment"][ToString@DIV[{"class"->"IntroLinkCell"}, {Recurse[] } ] ],
*)

{StyleBox, "GuideFunctionsSubsection"} :> Recurse[],

{Cell, "GuideAbstract"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    WithParameters[{"newline2br" -> False, "newline2div" -> True, "styleUsageMessageBreak" -> "<span class='br'>&nbsp;</span>"},P[{"class"->"intro"}, {Recurse[]} ]],
    DIV[{},{}]
  ]
],

{Cell, "GuideFunctionsSubsection"} :> Module[
  {flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    H3[
      If[HasOption[CellTags],
        {"id" -> StringReplace[
          ToString[Flatten[{GetOption[CellTags]}]],
          {"{" -> "", "}"->"", "$" -> "dollar"}]},
        {}
      ],
      {Recurse[]}
    ],
    DIV[{},{}]
  ]
],

{Cell, "GuideFunctionsSubsubsection"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    H4[{}, {Recurse[]} ],
    DIV[{},{}]
  ]
],


{Cell, "GuideDelimiterSubsection"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    XMLElement["div", {"class"->"GuideDelimiterSubsection"}, {XML`RawXML["<!-- -->"]} ],
    DIV[{},{}]
  ]
],

{Cell, "GuideDelimiter"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    XMLElement["div", {"class"->"GuideDelimiter"}, {XML`RawXML["<!-- -->"]} ],
    DIV[{},{}]
  ]
],

{Cell, "FeaturedExamplesGridCell"} :> Unevaluated[Sequence[]],

{Cell, "GuideMainTutorial" | "GuideTestDrive" | "GuideTitle" | "GuideFunctionGroupCaption"} :> 
Module[{flag = GetParameter["flag"]}, 
  If[flag =!= "RootGuidePage",
    DIV[{"class"->GetStyle[]}, {
    	Recurse[]
    	(*A[{"name" -> StringReplace[GetStyle[], {"Guide"->""} ]}, {Recurse[]}]*)
    	} ],
    XMLElement["span",{}, {Recurse[]}]
  ]
],

{CellGroupData, "GuideFunctionsSection"} :> Module[{gfstitle, fgstag},
	gfstitle=Flatten[{Transmogrify[First[SelectChildren[]]]}];
	fgstag = If[Length[Cases[gfstitle, XMLElement["a", {___}, {___}], Infinity]]>0, "span", "a"];
	XMLElement["div", {"class"->"expandWrap"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			If[fgstag==="span",
			XMLElement["span", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]]}, gfstitle],
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},gfstitle]], 
			If[fgstag==="span",
			XMLElement["span", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]]}, gfstitle],
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},gfstitle]]
			}], 
			XMLElement["div",{"class"->"expanded"}, 
				Flatten[{If[
					Length[Cases[First[SelectChildren[][[2]]], Cell[___, "GuideFunctionsSubsection", ___], Infinity]] > 0,
					 {}, 
					 XMLElement["div", {"class" -> "GuideDelimiterTop"}, {XML`RawXML["<!-- -->"]}]], Flatten[{Transmogrify[Rest[SelectChildren[]]]}]}]]}]}]],
			
{Cell, "GuideFunctionsSection"} :> Recurse[],

{CellGroupData, "GuideTutorialsSection" } :> XMLElement["div", {"class"->"expandWrap", "id"->"RelatedTutorials"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},{Localized["RelatedTutorialsSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},{Localized["RelatedTutorialsSectionText", GetParameter["language"]]}]}], 
			XMLElement["div",{"class"->"expanded"},{XMLElement["ul",{"class"->"functionList"},{Recurse[]}]}]}]}],

{Cell, "GuideTutorialsSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "GuideMoreAboutSection" } :> XMLElement["div", {"class"->"expandWrap", "id"->"RelatedGuides"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},{Localized["MoreAboutSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},{Localized["MoreAboutSectionText", GetParameter["language"]]}]}], 
			XMLElement["div",{"class"->"expanded"},{XMLElement["ul",{"class"->"functionList"},{Recurse[]}]}]}]}],

{Cell, "GuideMoreAboutSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "GuideRelatedLinksSection" } :> XMLElement["div", {"class"->"expandWrap", "id"->"RelatedLinks"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},{Localized["RelatedLinksSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},{Localized["RelatedLinksSectionText", GetParameter["language"]]}]}], 
			XMLElement["div",{"class"->"expanded"},{XMLElement["ul",{"class"->"functionList"},{Recurse[]}]}]}]}],

{Cell, "GuideRelatedLinksSection" } :> Unevaluated[Sequence[]],

{CellGroupData, "GuideTutorialCollectionSection" } :> XMLElement["div", {"class"->"expandWrap", "id"->"TutorialCollection"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},{Localized["TutorialCollectionSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},{Localized["TutorialCollectionSectionText", GetParameter["language"]]}]}], 
			XMLElement["div",{"class"->"expanded"},{XMLElement["ul",{"class"->"functionList"},{Recurse[]}]}]}]}],

{Cell, "GuideTutorialCollectionSection" } :> Unevaluated[Sequence[]],

(* {Cell, "RootMoreAboutDelimiter"} :> 
    DIV[{"class"->"GuideDelimiter"}, {Recurse[] } ],  *)
    
{CellGroupData, "GuideFeaturedExamplesSection"} :> Module[{cfp}, cfp = CreateFeaturedPage[GetParameter["featuredexampledata"]];
	XMLElement["div", {"class"->"expandWrap", "id"->"FeaturedExamples"},
	{XMLElement["div", {"class"->"expandOuter", "id"->"mycarousel"},{
		XMLElement["h2", {"class"->"expandInner"}, Flatten[{
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->""},{Localized["GuideFeaturedExamplesSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->""},{Localized["GuideFeaturedExamplesSectionText", GetParameter["language"]]}], 
			If[Length[cfp]>4, {XML`RawXML["<span class=\"jcarousel-scroll\"> 
							<a href=\"#\" class=\"buttons prev\"></a>
							<span class=\"page-out-of\"></span><span class=\"divider\">|</span><span class=\"total-pages\"></span>
							<a href=\"#\" class=\"buttons next\"></a>
							<a href=\"#\" class=\"buttons\" id=\"mycarousel-minimize\"></a>
							<a href=\"#FeaturedExamples\" id=\"mycarousel-show-all\"></a>
					</span>"]}, {}]
			}]], 
			XMLElement["div",{"class"->"expanded"},{XMLElement["div",{"class"->"viewport"},{XMLElement["ul",{"class"->"overview"},Flatten[{Map[Transmogrify,cfp]}]], XMLElement["div", {"class"->"clearboth"}, {XML`RawXML["<!-- -->"]}] }]}]}]}]],
			
{Cell, "GuideFeaturedExampleSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "FeaturedExampleLink"} :> XMLElement["li", {},{XMLElement["a", {"href"->Cases[SelectSelf[], Cell[a_, "FeaturedExampleLink", ___]:>a, Infinity][[1]]},{Recurse[]}]}],
			
{Cell, "FeaturedExampleLink"} :> Unevaluated[Sequence[]],

{Cell, "FeaturedExampleThumbnailImage"} :> transmogrifyImage[ToFileName[{"Files", GetFileNameBase[]},
    "featured_example"<>IncrementCounter["FeaturedExampleThumbnail"] <> ".png"], {"alt"->"carousel-img"<>GetCounter["FeaturedExampleThumbnail"]},
    Inline -> True, CropImage -> False, ImageFormat->"PNG"],

{Cell, "FeaturedExampleText"} :> XMLElement["span", {"class"->"carousel-caption"},{Recurse[]}],
    
{CellGroupData, "GuideLearningResourcesSection"} :> Module[{}, XMLElement["div", {"class"->"expandWrap closed", "id"->"LearningResources"},
	{XMLElement["div", {"class"->"expandOuter"},{
		XMLElement["h2", {"class"->"expandInner"}, {
			XMLElement["a", {"class"->"expand hide", "title"->Localized["show", GetParameter["language"]], "href"->"#"},{Localized["GuideLearningResourcesSectionText", GetParameter["language"]]}], 
			XMLElement["a", {"class"->"collapse", "title"->Localized["hide", GetParameter["language"]], "href"->"#"},{Localized["GuideLearningResourcesSectionText", GetParameter["language"]]}]}], 
			XMLElement["div",{"class"->"expanded"}, 
				{XMLElement["table", {"cellspacing" -> "0", "cellpadding" -> "0", "border" -> "0"}, {XMLElement["tr", {}, {Transmogrify[CreateLearningResources[SelectSelf[], ""]]}]}]}]}]}]],

{CellGroupData, "GuideFeaturedExamplesSection"} :> Unevaluated[Sequence[]],
{Cell, "GuideFeaturedExamplesSection"} :> Unevaluated[Sequence[]],

{Cell, "GuideLearningResourcesSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "GuideLearningResourcesLeftSection"} :> XMLElement["td", {"class" -> "resourcesLeft"}, {XMLElement["div", {}, Flatten[{Recurse[]}]]}],

{Cell, "GuideLearningResourcesLeftSection"} :> Unevaluated[Sequence[]],

{CellGroupData, "GuideLearningResourcesTutorialSection"}:> {Transmogrify[First[SelectChildren[]]], XMLElement["div", {"class"->"learning-links"}, {Transmogrify[Rest[SelectChildren[]]]}]},

{Cell, "GuideLearningResourcesTutorialSection"} :> XMLElement["h3", {"class"->"learning-tutorials"}, {Recurse[]}],

{Cell, "LearningResourceTutorial"} :> Recurse[], 

{CellGroupData, "GuideLearningResourcesHowToSection"} :> {Transmogrify[First[SelectChildren[]]], XMLElement["div", {"class"->"learning-links"}, {Transmogrify[Rest[SelectChildren[]]]}]}, 

{Cell, "GuideLearningResourcesHowToSection"} :> XMLElement["h3", {"class"->"learning-howtos"}, {Recurse[]}],

{Cell, "LearningResourceHowTo"} :> Recurse[],

{CellGroupData, "GuideLearningResourcesRightSection"} :> XMLElement["td", {"class" -> "resourcesRight"}, {Recurse[]}],

{Cell, "GuideLearningResourcesRightSection"} :> Unevaluated[Sequence[]],

{Cell, "SmallTitle"} :> XMLElement["p", {"class"->"smallTitle"}, {Recurse[]}],

{Cell, "ResourceTraining"} :> Recurse[],
{Cell, "ResourceVideos"} :> Recurse[],
{Cell, "ResourceDemonstrations"} :> Recurse[],
{Cell, "ResourceCommunity"} :> Recurse[],

{Cell, "RootMoreAboutDelimiter"} :> 
    DIV[{"class"->"RootMoreAboutDelimiter"}, {Recurse[] } ],

{Cell, "RootSectionCaption"} :> 
    DIV[{"class"->"GuideFunctionGroupCaption"}, {Recurse[] } ],

{Cell, "AlphabeticalListingAnchor"} :>
  DIV[
    {"class" -> "AlphabetListingJumpTo"},
    Map[
      XMLElement[
        "a",
        {"href" -> "#" <> StringReplace[#, "$" -> "dollar"]},
        {#}
      ]&,
      Cases[SelectSelf[], Rule[False, w_] :> w, Infinity]
    ]
  ],

{Cell, "GuideMoreAbout" | "GuideMoreAboutSub" | "GuideTutorial" | "GuideRelatedLinks" | "GuideTutorialCollection"} :>
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
	Li[{Recurse[]}],
    DIV[{"class"->"OtherItems"},  {
	UL[{"class"->GetStyle[]}, { 
  	Li[{ Recurse[] }]
 		 }]
		}]
	] ],
	

{Cell, "RootMoreAbout"} :> {},

{Cell, "GuideText"} :> 
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    P[{"class"->"singleFunction"},  { Recurse[] }],
    Module[{guidelinks,guidelinksNew678,class="left"},
      guidelinks = Cases[SelectSelf[], _ButtonBox, Infinity];
      (*guidelinksNew678 = Insert[Take[guidelinks, {-3, -2, -1}], " | ", 2];*)
      guidelinksNew678 = Riffle[Take[guidelinks, {-3, -1}], " | "];
      guidelinks=Join[Drop[guidelinks, -3], {guidelinksNew678}];
	  guidelinks=If[EvenQ@Length@guidelinks === True, guidelinks, Append[guidelinks, {}]];
	  guidelinks = Partition[guidelinks, 2];
	  guidelinks = Transpose[guidelinks];
      guidelinks = Map[
        XMLElement["ul", {"class" -> (class="right";#)&[class]}, {Map[Li[{}, {Transmogrify[#]}]&, #]}]&,
        guidelinks];
      XMLElement["div",{
          "class" -> "accordion-content "<> GetCounter["RootSection"]
        }, {
        XMLElement["tr",{}, {
        TD[{"class"->"rootLinkCol"}, {guidelinks[[1]]}],
        TD[{"class"->"rootLinkCol"}, {DeleteCases[guidelinks[[2]], XMLElement["li",{__},{}], Infinity]}]
        }]
      }]
    ]
]],

{Cell, "RootLinks" | "RootLink"} :> 
Module[{flag = GetParameter["flag"]},
  If[flag =!= "RootGuidePage",
    DIV[{"class"->"GuideText"},  { Recurse[] }],
    Module[{guidelinks,guidelinksNew678,class="left"},
      guidelinks = Cases[SelectSelf[], _ButtonBox, Infinity];

      guidelinks = GatherBy[guidelinks, #[[-1]] &];

      (*guidelinksNew678 = Insert[Take[guidelinks, {-3, -2, -1}], " | ", 2];*)
      guidelinksNew678 = Riffle[Take[guidelinks, {-3, -1}], " | "];
      guidelinks=Join[Drop[guidelinks, -3], {guidelinksNew678}];
	  guidelinks=If[EvenQ@Length@guidelinks === True, guidelinks, Append[guidelinks, {}]];
	  guidelinks = Partition[guidelinks, 2];
	  guidelinks = Transpose[guidelinks];
      guidelinks = Map[
        XMLElement["ul", {"class" -> (class="right";#)&[class]}, Flatten[{Map[XMLElement["li", {}, {Transmogrify[#]}]&, #]}]]&,
        guidelinks];
      guidelinks = DeleteCases[guidelinks, XMLElement["li",{},{}], Infinity];
      XMLElement["div",{
          "class" -> "accordion-content "<> (GetCounter["RootSection"]/.{"1"->"one","2"->"two","3"->"three","4"->"four","5"->"five","6"->"six","7"->"seven","8"->"eight"})
        }, Flatten[{guidelinks[[1]], guidelinks[[2]]}]]
    ]
]],

{Cell, "RootDarkLink"} :> Recurse[],

{Cell, "RootText"} :> 
Module[{rows},
  rows = First@First@SelectNearestDescendants[GridBox];
  DIV[{"class" -> "GuideText"}, {
    XMLElement["ul", {"class" -> "RootText"}, Flatten[{
      Map[
        XMLElement["li", {}, 
          If[MatchQ[#, {_, Cell[TextData[_List, ___], ___]}],
            Flatten@Map[
              If[MatchQ[#, {StyleBox[" \[FilledSmallSquare] ", ___]}],
                {" ", XMLElement["img",
                  {"src" -> "http://reference.wolfram.com/mathematicaImages/bullet.gif",
                   "style" -> "vertical-align: middle", "alt"->"bullet", "title"->"bullet"},
                  {}], " "},
                Transmogrify /@ #
              ]&,
              Split[#[[2, 1, 1]], Not[Apply[Or, Map[
                MatchQ[#, StyleBox[" \[FilledSmallSquare] ", ___]]&,
                {#1, #2}]]]&]
            ],
            (* Else *)
            {Transmogrify[#]}
        ]]&,
        Most[rows] ],
      XMLElement["li", {}, {
        Transmogrify[rows[[-1, 2]]] /. XMLElement["a", atr_, cnt_] :>
          XMLElement["a", atr, {
            XMLElement["span", {"style" -> "color:#454545"}, {
              StringDrop[cnt[[1]], -1] }],
            StringTake[cnt[[1]], -1]
          }]
      }]
    }] ]
  }]
],

{StyleBox, "InlineSeparator"} :>If[StringReplace[GetFileNameBase[], {".en"->"", ".ja"->""}]==="Mathematica",
Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}],
Span[{"class" -> "inlineBullet"}, {XML`RawXML["&nbsp;"], XML`RawXML["&#9642;"], XML`RawXML["&nbsp;"]}]
],


{StyleBox, "LeadingBullet"} :>
Span[{"class" -> "LeadingBullet"}, {
    Img[{"src" -> htmlBase<>"/mathematicaImages/bullet.gif", "width"->"4", "height"->"4", "style"->"vertical-align:middle;", "alt"->"bullet", "title"->"bullet"}] }],

{Cell, "InlineListing"} :>
DIV[{"class" -> "InlineListing"}, {Recurse[] }],
{Cell, "InlineListing3Col"} :>
DIV[{"class" -> "InlineListing3Col"}, {Recurse[] }],
{Cell, "InlineListing4Col"} :>
DIV[{"class" -> "InlineListing4Col"}, {Recurse[] }],


{Cell, "InlineGuideFunction"} :> 
Span[{"class"->"InlineGuideFunction"}, {Recurse[]} ],


{Cell, "InlineGuideFunctionListing"} :> 
Module[{ml},
  ml = Recurse[];
  If[(Length@{ml} === 1) && (Head@ml === String),
   ml = StringSplit[ml, RegularExpression["\\s+\\.\\s+"] ];
   ml = Span[{"class"->"InlineGuideFunction"}, 
          {XMLElement["span", {}, {#}] }]&/@ ml;
   ml = Riffle[ml,  
     Span[{"class" -> "inlineGuideFunctionListingBullet"}, 
       {" ", Img[{"src" -> htmlBase<>"/mathematicaImages/bullet-guide.gif", "width"->"6", "height"->"12", "style"->"vertical-align:middle;", "alt"->"bullet-guide", "title"->"bullet-guide"}],
       " "}]
     ] ];
  P[{"class"->"inlineFunctionList"}, Flatten@{ml}]
],

PaneSelectorBox:>Transmogrify@SelectSelf[][[1,2,2]]


},

IncludeXMLTransforms -> {{"HTML","Components","Includes.m"}},

DefaultParameters -> {
  "newline2br" -> True,
  "newline2div" -> False,
  "styleUsageMessageBreak" -> "</div>\n<div class='UsageMessage'>",
  "noop" -> {
    $CreateCompleteHTMLFile = False;
    $LanguageExtension = "en";
   }
},

CounterReassignments -> {{"Chapter", "Section"}, {"Section", "Subsection"}, {"Subsection", "Subsubsection"} }

]
